<?php

namespace Teknisa\SM\Service;

use Teknisa\SM\Factory\IntegracaoPmFactory;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\ConnectionCustom;
use Teknisa\Libs\Util\DateUtil;
use Zeedhi\Framework\Remote\FileUploader\Server;
use Teknisa\SM\Util\Procedure\CriaMnemonico;
use Teknisa\SM\Util\Procedure\FctGeraContrato;
use Teknisa\SM\Service\Sm00001000DocumentoPropostaService;
use Teknisa\SM\Factory\Sm00001000DocumentoPropostaFactory;
use Teknisa\SM\Util\Utilities;
use Zeedhi\Framework\Remote\cURLRequest;
use Zeedhi\Framework\DTO\Row;
use Teknisa\SM\Service\Sm00001000LancamentoPropostaService;
use Teknisa\SM\Factory\Sm00001000LancamentoPropostaFactory;
use Teknisa\SM\Service\Sm00004000ContratoService;
use Teknisa\SM\Factory\Sm00004000ContratoFactory;
use Teknisa\SM\Service\SmDocumentosService;
use Teknisa\SM\Util\Procedure\FctGerarParcelasFatura;

class IntegracaoPmService {
    
    const OPERADOR_TEKNISA = "000000000997";
    /**
     * @var IntegracaoPmFactory
     */
    protected $integracaoFactory;
    protected $newCode;
    protected $entityManager;
    protected $connection;
    protected $connectionCustom;
    protected $fileServerBaseUrl;
    protected $apiKeyMapping;
    protected $propostaService;
    protected $propostaFactory;
    protected $contratoFactory;
    protected $smDocumentosService;
    protected $idSolicitacaoAguardCli;
    protected $idSolicitacaoAguardProp;
    protected $idSolicitacaoAguardAprov;
    protected $idSolicitacaoLiberada;
    protected $clienteSignatarioPrestador;
    protected $zhFileUploader;
    
    public function __construct(EntityManager $entityManager, ConnectionCustom $connectionCustom, $fileServerBaseUrl, $apiKeyMapping, $idSolicitacaoAguardCli, $idSolicitacaoAguardProp, $idSolicitacaoAguardAprov, $idSolicitacaoLiberada, $clienteSignatarioPrestador) {
        $this->entityManager            = $entityManager;
        $this->connectionCustom         = $connectionCustom;
        $this->fileServerBaseUrl        = $fileServerBaseUrl;
        $this->apiKeyMapping            = $apiKeyMapping;
        
        // retirada das '' dos parâmetros. São enviados assim para evitar que o valor seja corrompido.
        $this->idSolicitacaoAguardCli   = ($idSolicitacaoAguardCli)   ?  explode("'", $idSolicitacaoAguardCli)[1]   : null;
        $this->idSolicitacaoAguardProp  = ($idSolicitacaoAguardProp)  ?  explode("'", $idSolicitacaoAguardProp)[1]  : null;
        $this->idSolicitacaoAguardAprov = ($idSolicitacaoAguardAprov) ?  explode("'", $idSolicitacaoAguardAprov)[1] : null;
        $this->idSolicitacaoLiberada    = ($idSolicitacaoLiberada)    ?  explode("'", $idSolicitacaoLiberada)[1]    : null;
        
        $this->clienteSignatarioPrestador = $clienteSignatarioPrestador;
    }
    
    public function prepararInserirProposta($nrorg, $cdoperador, $codCliente, $codProduto, $descricao, $emailInterlocutor,
                                            $nomeInterlocutor, $itens, $desconto, $parcelas = null, $isConnected = false,
                                            $nomeProduto = null, $issueId = null, $nrsoliserv = null, $valorTotalMan = null,
                                            $percDescontoMan = null, $valorDescontoMan = null, $valorTotalManComDesc = null) {
        if(!$isConnected) {
            $this->setConnection($nrorg);
            $this->integracaoFactory   = new IntegracaoPmFactory($this->connection);
            $this->newCode             = new NewCode($this->connection, "NOVO_CODIGO_SC");
        }
    
        $this->propostaFactory     = new Sm00001000LancamentoPropostaFactory($this->entityManager);
        $this->smDocumentosService = new SmDocumentosService($this->apiKeyMapping);
        
        $this->connection->beginTransaction();
        try {
            if (($nrsoliserv == null) && ($issueId == null)) {

                throw new \Exception("Não foi possível criar proposta de customização: Não foi encontrado ID da issue ou código de solicitação.");
            }
            else {
                if ($nrsoliserv != null) {

                    $params = array(
                        "NRSOLISERV" => $nrsoliserv,
                        "NRORG"      => $nrorg
                    );

                    $resultado = $this->integracaoFactory->getNrOrcamentoSolicitacao($params);

                    if (count($resultado) > 0){

                        $valOrcaSoli = $resultado[0]["NRORCAMENTO"];
                    }
                    else {
                        $valOrcaSoli = null;

                        $params = array(
                            "CDFILIAL"   => "9999",
                            "NRSOLISERV" => $nrsoliserv,
                            "NRORG"      => $nrorg
                        );
                        
                        $faseTramitacao = $this->integracaoFactory->getFaseTramitacao($params);
                    }
                }

                if ((($nrsoliserv != null) && ($valOrcaSoli == null)) || ($issueId != null)) {

                    if ((($nrsoliserv != null) && ($faseTramitacao == $this->idSolicitacaoAguardProp)) || ($issueId != null)) {

                        if ((($nrsoliserv != null) && ($codCliente !== $this->clienteSignatarioPrestador)) || ($issueId != null)) {

                            $nrseqinterlocutor = $this->getCodigoInterlocutor($emailInterlocutor, $nomeInterlocutor, $nrorg);
                            
                            $nrparcnegocio = $this->getNrparcnegocioCliente($codCliente, $nrorg);
                            if (empty($nrparcnegocio))
                                throw new \Exception("Número de Parceiro de Negócio do Cliente não foi encontrado.");
                            
                            $dadosModelo = $this->getDadosModeloPropostaCustomizacao($nrorg);
                            if (!empty($dadosModelo)) {
                                $nrorcamentomodelo = $dadosModelo["NRORCAMENTOMODELO"];
                                $idfaturacontr     = $dadosModelo["IDFATURACONTR"];
                                $cdmodpgto         = $dadosModelo["CDMODPGTO"];
                                $nrgrupofatura     = $dadosModelo["NRGRUPOFATURA"];
                                $qtdiasvalidade    = $dadosModelo["QTDIASVALIDADE"];
                                $qtdiascontrato    = $dadosModelo["QTDIASCONTRATO"];
                                
                                if(!$nomeProduto && $issueId) $nomeProduto = $this->getNomeProduto($codProduto, $nrorg);
                                else if(!$nomeProduto && $nrsoliserv) $nomeProduto = $this->getNomeProdutoSolicitacao($codProduto, $nrorg);
                                if (empty($nomeProduto)) {
                                    throw new \Exception("Produto não foi encontrado.");
                                }
                                
                                $dadosManutCusto = $this->getDadosManutencaoCustomizacao($nrorg);
                                if (!empty($dadosManutCusto)) {
                                    
                                    $servOrcaManutCusto = $dadosManutCusto["CDSERVORCA"];
                                    $nmOrcaManutCusto   = $dadosManutCusto["NMSERVORCA"];
                                    $percManutCusto     = $dadosManutCusto["VRPERCMANUCUSTOM"];
                                    $escopoManutCusto   = $dadosManutCusto["NRESCOPOMANUCUSTOM"];
                                    
                                    if (empty($escopoManutCusto) || (empty($percManutCusto) && empty($valorTotalMan))) {
                                        $dadosManutCusto = null;
                                    }
                                    //throw new \Exception("Percentual de manutenção da customização não foi encontrado.");
                                    //throw new \Exception("Escopo de Faturamento da manutenção da customização não foi encontrado.");
                                }

                                $configRegionalParceiro = $this->getIdiomaMoedaParceiro($nrparcnegocio);
                                $cdidiom = $configRegionalParceiro["CDIDIOM"];
                                $nrmoeda = $configRegionalParceiro["NRMOEDA"];
                                
                                $nrseqorcamento = $this->inserirProposta($nrorg, $cdoperador, $codCliente, $codProduto, $nomeProduto, $descricao,
                                    $nrseqinterlocutor, $emailInterlocutor, $nomeInterlocutor, $nrparcnegocio, $nrorcamentomodelo, $idfaturacontr,
                                    $cdmodpgto, $nrgrupofatura, $qtdiasvalidade, $qtdiascontrato, $issueId, $nrsoliserv, $cdidiom, $nrmoeda);
                                
                                //chama a procedure CRIA_MNEMONICO
                                $procedure = new CriaMnemonico($this->connection);
                                $procedure->executar($nrorg, "O", $nrseqorcamento);
                                
                                //inserir escopos
                                $escopos = $this->getEscoposModeloProposta($nrorcamentomodelo, $nrorg);
                                $this->inserirEscoposProposta($nrseqorcamento, $escopos, $nrorg, $cdoperador);

                                //inserir documentos
                                $documentos = $this->getDocumentosEscoposModeloProposta($nrorcamentomodelo, $nrorg);
                                $this->inserirDocumentosProposta($nrseqorcamento, $documentos, $nrorg, $cdoperador);
                                
                                //inserir escopos (Manutenção da Customização)
                                if (!empty($dadosManutCusto)) {
                                 
                                    $this->inserirEscoposPropostaManutCust($nrseqorcamento, $escopos, $escopoManutCusto, $nrorg, $cdoperador);
                                }

                                $nomeServicos = $this->getNomeServicos($itens, $nrorg);
                                
                                /*$paramsFaturamentoEscopo = null;
                                if (count($escopos) > 0) {
                                    $paramsFaturamentoEscopo = $this->getParamsFaturamentoEscopo($escopos[0]["NRTIPOORCAMENTO"], $nrorg);
                                }*/
                                
                                //inserir itens do JSON
                                $this->inserirItensProposta($nrseqorcamento, $nrorg, $cdoperador, $itens, $nomeServicos, $nrmoeda);
                                
                                //insere os registros de negociação de valores na proposta
                                $this->inserirNegociacaoValoresProposta($nrseqorcamento, $codCliente, $escopos, $itens, $nrorg, $cdoperador, $desconto, $parcelas);
                                
                                if (!empty($dadosManutCusto)) {
                                    //Inserir Manutenção de Customização
                                    $this->inserirManutencaoCutomizacao($nrseqorcamento, $nrorg, $cdoperador, $itens, $desconto, $servOrcaManutCusto, $nmOrcaManutCusto, $percManutCusto, $escopoManutCusto, $valorTotalMan, $nrmoeda);
                    
                                    //insere os registros de negociação de valores de manutenção de customização na proposta
                                    $this->inserirNegociacaoValoresManutencaoCutomizacao($nrseqorcamento, $codCliente, $escopoManutCusto, $servOrcaManutCusto, $nrorg, $cdoperador, $valorTotalMan, $percDescontoMan, $valorDescontoMan, $valorTotalManComDesc);
                                }
                                
                                //atualizar o valor total da proposta
                                $this->atualizarValorTotalProposta($nrseqorcamento, $nrorg);

                                if ($nrsoliserv != null) {
                                    $this->alterarSituacaoSolicitacao($nrsoliserv, $nrorg, '9999', $this->idSolicitacaoAguardProp, $this->idSolicitacaoAguardCli, $cdoperador, false);
                                }

                            } else {
                                throw new \Exception("Modelo de Proposta de customização não foi encontrado.");
                            }
                        } else {
                            $this->alterarSituacaoSolicitacao($nrsoliserv, $nrorg, '9999', $this->idSolicitacaoAguardProp, $this->idSolicitacaoAguardCli, $cdoperador, true);
                            $nrseqorcamento = null;
                            // throw new \Exception("Solicitação: " . $nrsoliserv . " | Fase de tramitação atualizada. Não foi criada proposta no SM (cliente Teknisa)");
                        }
                    } else {
                        throw new \Exception("Não foi possível criar proposta de customização: A solicitação informada não se encontra na fase de geração de proposta.");
                    }
                } else {
                    throw new \Exception("Não foi possível criar proposta de customização: Já existe uma proposta para a solicitação informada.  Código da proposta: ". $valOrcaSoli);
                } 
            }
            $this->connection->commit();
            return $nrseqorcamento;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    private function setConnection($nrorg) {
        $novoEntityManager = $this->connectionCustom->updateVpdConnection($nrorg);
        if ($novoEntityManager) {
            $this->entityManager = $novoEntityManager;
            $this->connection    = $novoEntityManager->getConnection();
        } else {
            $this->connection = $this->entityManager->getConnection();
        }
    }
    
    private function getDadosModeloPropostaCustomizacao($nrorg) {
        $params = array(
            "NRORG" => $nrorg
        );
        
        $dataset = $this->integracaoFactory->getDadosModeloPropostaCustomizacao($params);
        return count($dataset) > 0 ? $dataset[0] : null;
    }
    
    private function getCodigoInterlocutor($emailinterloc, $nminterlocutor, $nrorg) {
        $params = array(
            "EMAILINTERLOC"  => $emailinterloc,
            "NMINTERLOCUTOR" => $nminterlocutor,
            "NRORG"          => $nrorg
        );
        
        $dataset = $this->integracaoFactory->getCodigoInterlocutor($params);
        return count($dataset) > 0 ? $dataset[0]["NRSEQINTERLOCUTOR"] : null;
    }
    
    private function getNrparcnegocioCliente($cdcliente, $nrorg) {
        $params = array(
            "CDCLIENTE" => $cdcliente,
            "NRORG"     => $nrorg
        );
        
        $dataset = $this->integracaoFactory->getNrparcnegocioCliente($params);
        return count($dataset) > 0 ? $dataset[0]["NRPARCNEGOCIO"] : null;
    }
    
    private function inserirProposta($nrorg, $cdoperador, $codCliente, $codProduto, $nomeProduto, $descricao, $nrseqinterlocutor,
                                     $emailInterlocutor, $nomeInterlocutor, $nrparcnegocio, $nrorcamentomodelo, $idfaturacontr,
                                     $cdmodpgto, $nrgrupofatura, $qtdiasvalidade, $qtdiascontrato, $issueId = null, $nrsoliserv = null,
                                     $cdidiom = null, $nrmoeda = null) {
        
        $nrseqorcamento = $this->newCode->getCodeByTableName("NRSEQORCAMENTO", $nrorg, 1, 10, true);
        if (!$qtdiasvalidade) $qtdiasvalidade = 30;
        $dataAtual = DateUtil::getDataAtual(true);
        $dataValidade = DateUtil::adicionaIntervalo($dataAtual, $qtdiasvalidade);

        $dsreferenciaorca = $this->createReferencia($nomeProduto, $issueId, $nrsoliserv);

        $params = array(
            "NRSEQORCAMENTO"       => $nrseqorcamento,
            "NRPARCNEGOCIO"        => $nrparcnegocio,
            "NRVERSAOORCAMENTO"    => 1,
            "NRPARCTRANSPORTADORA" => null,
            "DTREFORCAMENTO"       => $dataAtual->format("d/m/Y"),
            "CDPRECBASESERV"       => null,
            "CDREDUCONT"           => null,
            "IDFATURAORCA"         => $idfaturacontr,
            "CDMODPGTO"            => $cdmodpgto,
            "NRPARCINTERLOCUTOR"   => null,
            "NRGRUPOFATURA"        => $nrgrupofatura,
            "DSREFERENCIAORCA"     => $dsreferenciaorca,
            "IDTRANSPORTADORA"     => "PROPRIA",
            "QTDIASVALIDADE"       => $qtdiasvalidade,
            "DTVALIDADE"           => $dataValidade->format("d/m/Y"),
            "QTDIASCONTRATO"       => $qtdiascontrato,
            "DSEMAIL"              => $emailInterlocutor,
            "NMPARCINTERLOCUTOR"   => $nomeInterlocutor,
            "NRTELEFONE"           => null,
            "DSOBSERVACAOORCA"     => $descricao,
            "IDGERACONTRATO"       => "NOVO",
            "IDSITUACAOORCAMENTO"  => "CONCLUIDO",
            "VRORCAMENTO"          => 0,
            "NRORG"                => $nrorg,
            "IDATIVO"              => "S",
            "NRORGINCLUSAO"        => $nrorg,
            "CDOPERINCLUSAO"       => $cdoperador,
            "DSINTERORCA"          => $nomeInterlocutor,
            "NRORCAMENTOMODELO"    => $nrorcamentomodelo,
            "DTACEITEORCA"         => null,
            "NRSEQINTERLOCUTOR"    => $nrseqinterlocutor,
            "CDOPERRESPONSAVEL"    => $cdoperador,
            "ISSUEID"              => $issueId,
            "NRSOLISERV"           => $nrsoliserv,
            "IDPRODUTO"            => $codProduto,
            "IDASSELETRDIGITAL"    => "N",
            "CDIDIOM"              => $cdidiom,
            "NRMOEDA"              => $nrmoeda
        );
        
        $this->integracaoFactory->inserirProposta($params);
        
        return $nrseqorcamento;
    }
    
    private function getEscoposModeloProposta($nrorcamentomodelo, $nrorg) {
        $params = array(
            "NRORCAMENTOMODELO" => $nrorcamentomodelo,
            "NRORG"             => $nrorg
        );
        
        return $this->integracaoFactory->getEscoposModeloProposta($params);
    }
    
    private function inserirEscoposProposta($nrseqorcamento, $escopos, $nrorg, $cdoperador) {
        foreach ($escopos as $escopo) {
            
            $nrrelacescopoproposta = $this->newCode->getCodeByTableName("FCT_RELACESCOPOPROPOSTA", $nrorg, 1, 10, true);
            
            $params = array(
                "NRRELACESCOPOPROPOSTA" => $nrrelacescopoproposta,
                "NRSEQORCAMENTO"        => $nrseqorcamento,
                "NRTIPOORCAMENTO"       => $escopo["NRTIPOORCAMENTO"],
                "NRSEQMODDOCFCT"        => null,
                "CDFILIAL"              => $escopo["CDFILIAL"],
                "NRORG"                 => $nrorg,
                "IDATIVO"               => "S",
                "NRORGINCLUSAO"         => $nrorg,
                "CDOPERINCLUSAO"        => $cdoperador
            );
            
            $this->integracaoFactory->inserirEscopoProposta($params);
        }
    }
    
    private function inserirEscoposPropostaManutCust($nrseqorcamento, $escopos, $escopoManutCusto, $nrorg, $cdoperador) {
        foreach ($escopos as $escopo) {
            
            if ($escopo["NRTIPOORCAMENTO"] != $escopoManutCusto){
            
                $nrrelacescopoproposta = $this->newCode->getCodeByTableName("FCT_RELACESCOPOPROPOSTA", $nrorg, 1, 10, true);
                
                $params = array(
                    "NRRELACESCOPOPROPOSTA" => $nrrelacescopoproposta,
                    "NRSEQORCAMENTO"        => $nrseqorcamento,
                    "NRTIPOORCAMENTO"       => $escopoManutCusto,
                    "NRSEQMODDOCFCT"        => null,
                    "CDFILIAL"              => '9990',
                    "NRORG"                 => $nrorg,
                    "IDATIVO"               => "S",
                    "NRORGINCLUSAO"         => $nrorg,
                    "CDOPERINCLUSAO"        => $cdoperador
                );
                
                $this->integracaoFactory->inserirEscopoProposta($params);
            }
        }
    }
    
    private function getDocumentosEscoposModeloProposta($nrorcamentomodelo, $nrorg) {
        $params = array(
            "NRORCAMENTOMODELO" => $nrorcamentomodelo,
            "NRORG"             => $nrorg
        );
        
        return $this->integracaoFactory->getDocumentosEscoposModeloProposta($params);
    }
    
    private function inserirDocumentosProposta($nrseqorcamento, $documentos, $nrorg, $cdoperador) {
        foreach ($documentos as $documento) {
            $nome    = $documento["NMARQUIVO"];
            $caminho = $documento["DSCAMINHOARQ"];
            $tipo    = $documento["DSTIPO"];
            $tamanho = $documento["VRTAMANHO"];            

            if (!empty($caminho)) {
                $caminhoArq = $this->fileServerBaseUrl . $caminho;
                
                $arquivoDoc = $this->smDocumentosService->getDocumentoFileServer($caminhoArq, $nrorg);
                if ($arquivoDoc !== false) {
                    $arquivoDoc = base64_encode($arquivoDoc);
                    
                    /* 2 - Anexar o documento novamente */
                    $this->setZhFileUploader($nrorg);
                    $anexoNovo = $this->anexaOrcamentoDoc($arquivoDoc, $nome, $tipo, $tamanho);
                    
                    $nomeNovo    = $anexoNovo["name"];
                    $caminhoNovo = $anexoNovo["path"];
                    $tamanhoNovo = $anexoNovo["size"];
                    $tamanhoNovo = $tamanhoNovo / 1024;
                    $tipoNovo    = $anexoNovo["type"];

                    $nrseqorcadoc = $this->newCode->getCodeByTableName("NRSEQORCADOC", $nrorg, 1, 10, true);
                
                    $params = array(
                        "NRSEQORCADOC"        => $nrseqorcadoc,
                        "NRSEQORCAMENTO"      => $nrseqorcamento,
                        "NRSEQMODDOCFCT"      => $documento["NRSEQMODDOCFCT"],
                        "IDIDIOMA"            => null,
                        "TXMODELOORCADOC"     => null,
                        "IDPADRAOORCADOC"     => null,
                        "NRORG"               => $nrorg,
                        "IDATIVO"             => "S",
                        "NRORGINCLUSAO"       => $nrorg,
                        "CDOPERINCLUSAO"      => $cdoperador,
                        "NMARQUIVO"           => $nomeNovo,
                        "DSCAMINHOARQ"        => $caminhoNovo,
                        "DSTIPO"              => $tipoNovo,
                        "VRTAMANHO"           => $tamanhoNovo,
                        "IDSITUACAOORCAMENTO" => null,
                        "NRTIPOORCAMENTO"     => $documento["NRTIPOORCAMENTO"]
                    );
                    
                    $this->integracaoFactory->inserirDocumentosProposta($params);

                    // insere as situacoes de proposta habilitadas para esde documento
                    $situacoes = explode(",", $documento["IDSITUACAOORCAMENTO"]);
                    $this->inserirSituacaoDocumentoProposta($situacoes, $nrseqorcadoc, $nrorg, $cdoperador);

                    // busca os anexos por idioma
                    $this->inserirAnexosIdioma($documento, $nrseqorcadoc, $nomeNovo, $caminhoNovo, $tamanhoNovo, $tipoNovo, $nrorg, $cdoperador);
                }
            }
        }
    }

    public function inserirSituacaoDocumentoProposta(array $situacoes, $nrseqorcadoc, $nrorg, $cdoperador) {
        $params = [
            "NRORCAMENTODOCSITU"  => null,
            "NRSEQORCADOC"        => $nrseqorcadoc,
            "IDSITUACAOORCAMENTO" => null,
            "IDATIVO"             => "S",
            "NRORG"               => $nrorg,
            "NRORGINCLUSAO"       => $nrorg,
            "CDOPERINCLUSAO"      => $cdoperador
        ];

        foreach ($situacoes as $situacao) {
            $params["NRORCAMENTODOCSITU"] = $this->newCode->getCodeByTableName("FCT_ORCAMENTODOCSITU", $nrorg, 1, 10, true);
            $params["IDSITUACAOORCAMENTO"] = $situacao;
            $this->propostaFactory->inserirSituacaoDocumentoProposta($params);
        }
    }
    
    public function inserirAnexosIdioma($doc, $nrseqorcadoc, $nomeArqivoOrcaDoc, $caminhoArqivoOrcaDoc, $tamanhoArqivoOrcaDoc, $tipoArqivoOrcaDoc, $nrorg, $cdoperador) {
        $anexosIdioma = $this->getAnexosModeloDocumentoPorIdioma($doc["NRSEQMODDOCFCT"]);
        foreach ($anexosIdioma as $anexo) {
            if (!empty($anexo["DSCAMINHOARQ"])) {
                if ($anexo["DSCAMINHOARQ"] == $doc["DSCAMINHOARQ"]) {
                    $nomeAnexo    = $nomeArqivoOrcaDoc;
                    $caminhoAnexo = $caminhoArqivoOrcaDoc;
                    $tipoAnexo    = $tipoArqivoOrcaDoc;
                    $tamanhoAnexo = $tamanhoArqivoOrcaDoc;
                }
                else {
                    $arquivoDocIdioma = $this->smDocumentosService->getDocumentoFileServer($this->fileServerBaseUrl . $anexo["DSCAMINHOARQ"], $nrorg);
                    if ($arquivoDocIdioma !== false) {
                        $arquivoDocIdioma = base64_encode($arquivoDocIdioma);

                        $anexoNovo = $this->anexaOrcamentoDoc($arquivoDocIdioma, $anexo["NMARQUIVO"], $anexo["DSTIPO"], $anexo["VRTAMANHO"]);
                        $nomeAnexo    = $anexoNovo["name"];
                        $caminhoAnexo = $anexoNovo["path"];
                        $tipoAnexo    = $anexoNovo["type"];
                        $tamanhoAnexo = $anexoNovo["size"] / 1024;
                    }
                }
                
                $nrorcamentodocidioma = $this->newCode->getCodeByTableName("FCT_ORCAMENTODOCIDIOMA", $nrorg, 1, 10, true);
                $params = array(
                    "NRORCAMENTODOCIDIOMA" => $nrorcamentodocidioma,
                    "NRSEQORCADOC"         => $nrseqorcadoc,
                    "CDIDIOM"              => $anexo["CDIDIOM"],
                    "NMARQUIVO"            => $nomeAnexo,
                    "DSCAMINHOARQ"         => $caminhoAnexo,
                    "DSTIPO"               => $tipoAnexo,
                    "VRTAMANHO"            => $tamanhoAnexo,
                    "IDATIVO"              => "S",
                    "NRORG"                => $nrorg,
                    "NRORGINCLUSAO"        => $nrorg,
                    "CDOPERINCLUSAO"       => $cdoperador
                );

                $this->propostaFactory->inserirAnexosDocumentoPropostaPorIdioma($params);
            }
        }
    }

    public function getAnexosModeloDocumentoPorIdioma($nrseqmoddocfct) {
        $params = array(
            "NRSEQMODDOCFCT" => $nrseqmoddocfct
        );

        return $this->propostaFactory->getAnexosModeloDocumentoPorIdioma($params);
    }
    
    private function getNomeServicos($itens, $nrorg) {
        $codServicos = array();
        foreach ($itens as $item) {
            $codServicos[] = $item["id"];
        }
        
        $codServicos = implode(",", $codServicos);
        $dataset = $this->integracaoFactory->getNomeServicos($codServicos, $nrorg);

        $servicos = array();
        foreach ($dataset as $row) {
            $servicos[$row["CDSERVORCA"]] = $row["NMSERVORCA"];
        }
        
        return $servicos;
    }
    
    private function inserirItensProposta($nrseqorcamento, $nrorg, $cdoperador, $itens, $nomeServicos, $nrmoeda = null) {
        foreach ($itens as $item) {
            
            $nritemorcamento = $this->newCode->getCodeByTableName("NRITEMORCAMENTO", $nrorg, 1, 10, true);
            
            $valorUnitario         = $item["unitaryValue"];
            $valorTotalSemDesconto = $item["totalValue"];
            $quantidade            = $item["quantity"];
            $nomeMedida            = $item["measurement"];
            
            $tipoDesconto       = "N";
            $percentualDesconto = null;
            $valorTotal         = $valorTotalSemDesconto;
            
            $nrgrupofatura         = null;
            $idfaturaorca          = null;
            $cdmodpgto             = null;
            $qtparcelasfatu        = null;
            $nrdiavencparcelasfatu = null;
            /*if (!empty($paramsFaturamentoEscopo)) {
                $nrgrupofatura         = $paramsFaturamentoEscopo["NRGRUPOFATURA"];
                $idfaturaorca          = $paramsFaturamentoEscopo["IDFATURACONTR"];
                $cdmodpgto             = $paramsFaturamentoEscopo["CDMODPGTO"];
                $qtparcelasfatu        = $paramsFaturamentoEscopo["QTPARCELASFATU"];
                $nrdiavencparcelasfatu = $paramsFaturamentoEscopo["NRDIAVENCPARCELASFATU"];
            }*/

            $params = array(
                "DSTPMEDIDASERV" => trim($nomeMedida),
                "NRORG"          => $nrorg
            );
            $dataset = $this->integracaoFactory->getTipoMedidaPeloNome($params);
            if (count($dataset) > 0) {
                $codigoMedida = $dataset[0]["CDTPMEDIDASERV"];
                $quantidadeMedida = $quantidade;
                $quantidade = 1;
                $usaMedida = "S";
            }
            else {
                $codigoMedida = null;
                $quantidadeMedida = null;
                $usaMedida = "N";
            }
            
            $params = array(
                "NRITEMORCAMENTO"       => $nritemorcamento,
                "NRSEQORCAMENTO"        => $nrseqorcamento,
                "CDSERVORCA"            => $item["id"],
                "NRITEMORCAMENTOPAI"    => null,
                "NMITEMORCAMENTO"       => $nomeServicos[$item["id"]],
                "QTITEMORCAMENTO"       => $quantidade,
                "CDTPMEDIDASERV"        => $codigoMedida,
                "QTMEDIDAORCA"          => $quantidadeMedida,
                "NRGRUPOFATURA"         => $nrgrupofatura,
                "IDFATURAORCA"          => $idfaturaorca,
                "CDMODPGTO"             => $cdmodpgto,
                "QTPARCELASFATU"        => $qtparcelasfatu,
                "NRDIAVENCPARCELASFATU" => $nrdiavencparcelasfatu,
                "DTINICFATORC"          => null,
                "VRUNITITEMORCA"        => $valorUnitario,
                "VRUNITITEMORCALOCA"    => null,
                "VRTOTALITEMORCA"       => $valorTotal,
                "TXFORMULAITORCA"       => null,
                "IDTRANSPORTADORA"      => "PROPRIA",
                "QTDIASCONTRATO"        => null,
                "IDITEMOBRIGORCA"       => "N",
                "IDSOMASUBITENS"        => "N",
                "IDSOMANOITEMPAI"       => "N",
                "IDSOMANOTOTALORCA"     => "N",
                "IDIMPRESSAOORCA"       => "S",
                "IDPRECOLOCACAO"        => "FIXO",
                "NRORG"                 => $nrorg,
                "IDATIVO"               => "S",
                //"DTINCLUSAO"            => null,
                "NRORGINCLUSAO"         => $nrorg,
                "CDOPERINCLUSAO"        => $cdoperador,
                "IDMULTMEDVRTOT"        => $usaMedida,
                "IDBLOQALTORC"          => "N",
                "IDADICITEMORC"         => "N",
                "IDADICCOBRITEMORC"     => "N",
                "VRITEMSEMDESC"         => $valorTotalSemDesconto,
                "VRPERCDESCORCA"        => $percentualDesconto,
                "IDTPDESCORCA"          => $tipoDesconto,
                "NRTIPOORCAMENTO"       => null,
                "NRMOEDA"               => $nrmoeda
            );
            
            $this->integracaoFactory->inserirItensProposta($params);
        }
    }
    
    
    private function getDadosManutencaoCustomizacao($nrorg) {
        $params = array(
            "NRORG" => $nrorg
        );
        
        $dataset = $this->integracaoFactory->getDadosManutencaoCustomizacao($params);
        return count($dataset) > 0 ? $dataset[0] : null;
    }
    
    private function inserirManutencaoCutomizacao($nrseqorcamento, $nrorg, $cdoperador, $itens, $desconto, $servOrcaManutCusto, $nmOrcaManutCusto, $percManutCusto, $escopoManutCusto, $valorTotalMan, $nrmoeda = null) {
        if (empty($valorTotalMan)) {
            $valorCustomizacao = 0;
            foreach ($itens as $item) {
                $valorCustomizacao = $valorCustomizacao + $item["totalValue"];
            }
            
            /*Comentado pois o Túlio disse que o valor da manutenção deve ser calculado a partir do valor cheio da customização (sem desconto)
            if (isset($desconto["percent"])) {
                $percDesconto = $desconto["percent"];
                if ($percDesconto > 0) {
                    $valorCustomizacao = $valorCustomizacao - round($valorCustomizacao * $percDesconto / 100, 2);
                }
            }*/
            $valorManutCusto = $valorCustomizacao * $percManutCusto / 100;
        }
        else {
            $valorManutCusto = $valorTotalMan;
        }
        
        $quantidade            = 1;
        $tipoDesconto          = "N";
        $percentualDesconto    = null;
        $nrgrupofatura         = null;
        $idfaturaorca          = null;
        $cdmodpgto             = null;
        $qtparcelasfatu        = null;
        $nrdiavencparcelasfatu = null;
        $codigoMedida          = 11;
        $quantidadeMedida      = 1;
        $usaMedida             = "S";
        
        $nritemorcamento = $this->newCode->getCodeByTableName("NRITEMORCAMENTO", $nrorg, 1, 10, true);
        $params = array(
            "NRITEMORCAMENTO"       => $nritemorcamento,
            "NRSEQORCAMENTO"        => $nrseqorcamento,
            "CDSERVORCA"            => $servOrcaManutCusto,
            "NRITEMORCAMENTOPAI"    => null,
            "NMITEMORCAMENTO"       => $nmOrcaManutCusto,
            "QTITEMORCAMENTO"       => $quantidade,
            "CDTPMEDIDASERV"        => $codigoMedida,
            "QTMEDIDAORCA"          => $quantidadeMedida,
            "NRGRUPOFATURA"         => $nrgrupofatura,
            "IDFATURAORCA"          => $idfaturaorca,
            "CDMODPGTO"             => $cdmodpgto,
            "QTPARCELASFATU"        => $qtparcelasfatu,
            "NRDIAVENCPARCELASFATU" => $nrdiavencparcelasfatu,
            "DTINICFATORC"          => null,
            "VRUNITITEMORCA"        => $valorManutCusto,
            "VRUNITITEMORCALOCA"    => null,
            "VRTOTALITEMORCA"       => $valorManutCusto,
            "TXFORMULAITORCA"       => null,
            "IDTRANSPORTADORA"      => "PROPRIA",
            "QTDIASCONTRATO"        => null,
            "IDITEMOBRIGORCA"       => "N",
            "IDSOMASUBITENS"        => "N",
            "IDSOMANOITEMPAI"       => "N",
            "IDSOMANOTOTALORCA"     => "N",
            "IDIMPRESSAOORCA"       => "S",
            "IDPRECOLOCACAO"        => "FIXO",
            "NRORG"                 => $nrorg,
            "IDATIVO"               => "S",
            "NRORGINCLUSAO"         => $nrorg,
            "CDOPERINCLUSAO"        => $cdoperador,
            "IDMULTMEDVRTOT"        => $usaMedida,
            "IDBLOQALTORC"          => "N",
            "IDADICITEMORC"         => "N",
            "IDADICCOBRITEMORC"     => "N",
            "VRITEMSEMDESC"         => $valorManutCusto,
            "VRPERCDESCORCA"        => $percentualDesconto,
            "IDTPDESCORCA"          => $tipoDesconto,
            "NRTIPOORCAMENTO"       => $escopoManutCusto,
            "NRMOEDA"               => $nrmoeda
        );
        
        $this->integracaoFactory->inserirItensProposta($params);
    }
    
    public function anexaOrcamentoDoc($arquivoDoc, $nome, $tipo, $tamanho) {
         $fileData = array(
            'b64File' => 'data:'.$tipo.';base64,'.$arquivoDoc,
            'name' => $nome,
            'size' => $tamanho * 1024,
            'type' => $tipo,
            'webkitRelativePath' => ''
        );

        $row = new Row(array('FILE' => array($fileData)));
        $this->zhFileUploader->uploadFilesInRow($row, 'FILE', 'anexo_documento_proposta');
        
        return $row["FILE"][0];
    }
    
    private function atualizarValorTotalProposta($nrseqorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );
        
        $this->integracaoFactory->atualizarValorTotalProposta($params);
    }
    
    private function getNomeProduto($codProduto, $nrorg) {
        $params = array(
            "IDELEMENTO" => "P_" . $codProduto,
            "NRORG"      => $nrorg
        );
        
        $dataset = $this->integracaoFactory->getNomeProduto($params);
        return count($dataset) > 0 ? $dataset[0]["NMSERVORCA"] : null;
    }
    
    //aqui
    public function aceitarProposta($nrseqorcamento, $nrorg, $cdoperador) {
        $this->setConnection($nrorg);
        $this->integracaoFactory = new IntegracaoPmFactory($this->connection);
        $idSituacao = $this->getSituacaoProposta($nrseqorcamento, $nrorg);
        if($idSituacao != "ACEITO"){
            $this->propostaFactory = new Sm00001000LancamentoPropostaFactory($this->entityManager);
            $this->smDocumentosService = new SmDocumentosService($this->apiKeyMapping);
            $this->setZhFileUploader($nrorg);
            
            $this->connection->beginTransaction();
            
            try {
                $params = array(
                    "CDOPERULTATU"   => $cdoperador,
                    "NRORGULTATU"    => $nrorg,
                    "NRSEQORCAMENTO" => $nrseqorcamento,
                    "NRORG"          => $nrorg,
                );
                
                $this->integracaoFactory->atualizarPropostaAceite($params);
                
                $procedure = new FctGeraContrato($this->connection);
                $procedure->executar($nrorg, $nrseqorcamento, $cdoperador);
                
                $this->anexarDocumentosContratosGerados($nrseqorcamento, $nrorg);
                
                $this->connection->commit();
            } catch (\Exception $e) {
                $this->connection->rollback();
                throw $e;
            }    
        }
    }
    
    private function anexarDocumentosContratosGerados($nrseqorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG" => $nrorg
        );
        
        $dataset = $this->propostaFactory->getDocumentosContratosGerados($params);
        foreach ($dataset as $row) {
            $nomeArquivo   = $row["NMARQUIVO"];
            $tipo          = $row["DSTIPO"];
            $tamanhoBytes  = $row["VRTAMANHO"] * 1024;
            $urlFileServer = $row["DSCAMINHOARQ"];
            
            if (!empty($urlFileServer)) {
                //$conteudo = @file_get_contents($this->fileServerBaseUrl . $urlFileServer);
                $conteudo = $this->smDocumentosService->getDocumentoFileServer($this->fileServerBaseUrl . $urlFileServer, $nrorg);
                if ($conteudo !== false) {
                    $file = Utilities::uploadFileServer($this->zhFileUploader, "anexo_documento_contrato", $nomeArquivo, $conteudo, $tamanhoBytes, $tipo);
                    $caminhoNovo = $file["path"];
                    
                    $params = array(
                        "DSCAMINHOARQ"  => $caminhoNovo,
                        "NRSEQCONTDOC"  => $row["NRSEQCONTDOC"],
                        "NRSEQCONTRATO" => $row["NRSEQCONTRATO"],
                        "NRORG"         => $nrorg
                    );
                    
                    $this->propostaFactory->atualizarCaminhoDocumentoContrato($params);
                }
            }
        }
    }
    
    public function cancelarProposta($nrseqorcamento, $nrorg, $cdoperador) {
        $this->setConnection($nrorg);
        $this->integracaoFactory = new IntegracaoPmFactory($this->connection);
        
        $this->atualizarPropostaCancelamento($nrseqorcamento, $cdoperador, $nrorg);
    }
    
    public function gerarDocumentosProposta($nrseqorcamento, $nrorg) {
        $this->setConnection($nrorg);
        $this->integracaoFactory   = new IntegracaoPmFactory($this->connection);
        $this->smDocumentosService = new SmDocumentosService($this->apiKeyMapping);
        
        $documentoFactory = new Sm00001000DocumentoPropostaFactory($this->entityManager);
        $documentoService = new Sm00001000DocumentoPropostaService($this->entityManager, $documentoFactory, $this->fileServerBaseUrl, $this->smDocumentosService);
        
        $idSituacao = $this->getSituacaoProposta($nrseqorcamento, $nrorg);
        
        $caminhoArquivos = $documentoService->gerarDocumentoProposta($nrseqorcamento, 0, $nrorg, $idSituacao, "P");
        
        return $caminhoArquivos;
    }
    
    private function getSituacaoProposta($nrseqorcamento, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG"          => $nrorg
        );
        
        $dataset = $this->integracaoFactory->getSituacaoProposta($params);
        return $dataset[0]["IDSITUACAOORCAMENTO"];
    }
    
    public function anexarDocumentosProposta($caminhoArquivos, $nrorg) {
        $this->setZhFileUploader($nrorg);
        
        $links = array();
        $customFolder = "anexo_documento_proposta";
        
        foreach ($caminhoArquivos as $caminho) {
            $file = Utilities::uploadArquivoFileServer($this->zhFileUploader, $customFolder, $caminho);
            $links[] = $file["path"];
        }
        
        $linksCompletos = $this->completarUrlLinks($links);
        return $linksCompletos;
    }
    
    public function completarUrlLinks(array $links) {
        foreach ($links as &$link) {
            $link = $this->fileServerBaseUrl . $link;
        }
        unset($link);
        
        return $links;
    }
    
    private function setZhFileUploader($nrorg) {
        if (isset($this->apiKeyMapping[$nrorg])) {
            $key = $this->apiKeyMapping[$nrorg];
            
            $explodeFileServerUrl = explode("/", $this->fileServerBaseUrl);
            
            $fileServerBackendIndexUrl = $explodeFileServerUrl[0] . "//" . $explodeFileServerUrl[2] . "/index.php";
            
            $this->zhFileUploader = new Server(new cURLRequest($fileServerBackendIndexUrl), $key);
        }
        else {
            throw new \Exception("Erro ao anexar documento no FileServer: Não existe chave de API parametrizada para o número de organização informado.");
        }
    }

    private function inserirNegociacaoValoresProposta($nrseqorcamento, $codCliente, array $escopos, array $itens, $nrorg, $cdoperador, $desconto, $parcelas = null) {
        $codEscopos = array();
        foreach ($escopos as $escopo) {
            $codEscopos[] = $escopo["NRTIPOORCAMENTO"];
        }
        $codEscopos = implode(",", $codEscopos);
        
        $codServicos = array();
        foreach ($itens as $item) {
            $codServicos[] = $item["id"];
        }
        $codServicos = implode(",", $codServicos);

        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG" => $nrorg
        );

        $paramFatItemNegociacao = $this->integracaoFactory->getParametrosFaturamentoParaItensNegociacaoProposta($params, $codEscopos, $codServicos);
        $itemNegoAnterior = array();
        $codNegociacoesInseridas = array();
        $qtdParcelasEscopoNegociacao = array();

        foreach ($paramFatItemNegociacao as $itemNego) {
            if ($itemNego["CDFILIALFATURA"] == null) {
                throw new \Exception('Operação bloqueada. <br/><br/>O parâmetro "Unidade para Faturamento" não foi encontrado para o item "' . $itemNego["NMITEMORCAMENTO"] . '". Favor conferir a parametrização de faturamento do Escopo.');
            }
            if ($itemNego["CDTIPOOPERFATURA"] == null) {
                throw new \Exception('Operação bloqueada. <br/><br/>O parâmetro "Tipo de Operação da Nota Fiscal" não foi encontrado para o item "' . $itemNego["NMITEMORCAMENTO"] . '". Favor conferir a parametrização de faturamento do Escopo.');
            }
            if ($itemNego["CDSERIENF"] == null) {
                throw new \Exception('Operação bloqueada. <br/><br/>O parâmetro "Série da Nota Fiscal" não foi encontrado para o item "' . $itemNego["NMITEMORCAMENTO"] . '". Favor conferir a parametrização de faturamento do Escopo.');
            }

            $competencia = array();
            if ($itemNego["IDTPFATURA"] != "P") {
                $params = array(
                    "NRTIPOORCAMENTO" => $itemNego["NRTIPOORCAMENTO"],
                    "CDSERVORCA"      => $itemNego["CDSERVORCA"],
                    "NRORG"           => $nrorg
                );
                $competencia = $this->propostaFactory->getCompetenciaItemNegociacao($params);
            }

            $itemNego["COMPETENCIA"] = print_r($competencia, true);
            
            if (Sm00001000LancamentoPropostaService::itensNegociacaoSaoDiferentes($itemNego, $itemNegoAnterior)) {
                $nrnegociacaoproposta = $this->newCode->getCodeByTableName("FCT_NEGOCIACAOPROPOSTA", $nrorg, 1, 10, true);
                $params = array(
                    "NRNEGOCIACAOPROPOSTA" => $nrnegociacaoproposta,
                    "NRSEQORCAMENTO"       => $nrseqorcamento,
                    "NRTIPOORCAMENTO"      => $itemNego["NRTIPOORCAMENTO"],
                    "IDTPFATURA"           => $itemNego["IDTPFATURA"],
                    "CDFILIALFATURA"       => $itemNego["CDFILIALFATURA"],
                    "NRPARCFATURA"         => $itemNego["NRPARCFATURA"],
                    "CDTIPOOPERFATURA"     => $itemNego["CDTIPOOPERFATURA"],
                    "CDSERIENF"            => $itemNego["CDSERIENF"],
                    "CDEMPRESA"            => $itemNego["CDEMPRESA"],
                    "CDINSCESTA"           => $itemNego["CDINSCESTA"],
                    "CDINSCMUNI"           => $itemNego["CDINSCMUNI"],
                    "VRTOTITEM"            => 0,
                    "VRPERCDESCITEM"       => null,
                    "VRDESCITEM"           => null,
                    "VRTOTITEMCOMDESC"     => 0,
                    "IDATIVO"              => "S",
                    "NRORG"                => $nrorg,
                    "NRORGINCLUSAO"        => $nrorg,
                    "CDOPERINCLUSAO"       => $cdoperador,
                    "IDPREVCOBRFUT"        => $itemNego["IDPREVCOBRFUT"],
                    "CDTRIBPRODPADR"       => $itemNego["CDTRIBPRODPADR"]
                );
                $this->propostaFactory->inserirNegociacaoProposta($params);
                
                $codNegociacoesInseridas[] = $nrnegociacaoproposta;
                
                if ($itemNego["IDTPFATURA"] == "P") {
                    $qtdParcelasEscopoNegociacao[$nrnegociacaoproposta] = $itemNego["QTPARCELASFATUESC"];
                }

                foreach ($competencia as $comp) {
                    $nrcompetencianegoprop = $this->newCode->getCodeByTableName("FCT_COMPETENCIANEGOPROP", $nrorg, 1, 10, true);
                    $params = array(
                        "NRCOMPETENCIANEGOPROP" => $nrcompetencianegoprop,
                        "NRNEGOCIACAOPROPOSTA"  => $nrnegociacaoproposta,
                        "NRDIACOMPETINI"        => $comp["NRDIACOMPETINI"],
                        "NRDIACOMPETFIN"        => $comp["NRDIACOMPETFIN"],
                        "IDMESCOMPETINI"        => $comp["IDMESCOMPETINI"],
                        "IDMESCOMPETFIN"        => $comp["IDMESCOMPETFIN"],
                        "QTDIASAPURACAO"        => $comp["QTDIASAPURACAO"],
                        "IDTPCALCVENC"          => $comp["IDTPCALCVENC"],
                        "NRDIACALCVENC"         => $comp["NRDIACALCVENC"],
                        "IDMESCALCVENC"         => $comp["IDMESCALCVENC"],
                        "QTDIASFATUVENC"        => $comp["QTDIASFATUVENC"],
                        "IDATIVO"               => "S",
                        "NRORG"                 => $nrorg,
                        "NRORGINCLUSAO"         => $nrorg,
                        "CDOPERINCLUSAO"        => $cdoperador
                    );
                    $this->propostaFactory->inserirCompetenciaNegociacaoProposta($params);
                }                
            }
            
            if (is_string($itemNego["VRTOTALITEMORCA"])) {
                if (strpos($itemNego["VRTOTALITEMORCA"], ',') !== false) {
                    $itemNego["VRTOTALITEMORCA"] = (float) str_replace(',', '.', str_replace(".", "", $itemNego["VRTOTALITEMORCA"]));
                }
            }

            $nritnegociacaoproposta = $this->newCode->getCodeByTableName("FCT_ITNEGOCIACAOPROPOSTA", $nrorg, 1, 10, true);
            $params = array(
                "NRITNEGOCIACAOPROPOSTA" => $nritnegociacaoproposta,
                "NRNEGOCIACAOPROPOSTA"   => $nrnegociacaoproposta,
                "NRITEMORCAMENTO"        => $itemNego["NRITEMORCAMENTO"],
                "NRITEMORCAMENTOPAI"     => $itemNego["NRITEMORCAMENTOPAI"],
                "VRTOTITEM"              => $itemNego["VRTOTALITEMORCA"],
                "VRPERCDESCITEM"         => null,
                "VRDESCITEM"             => null,
                "VRTOTITEMCOMDESC"       => $itemNego["VRTOTALITEMORCA"],
                "CDSERVORCA"             => $itemNego["CDSERVORCA"],
                "CDPRODFATURA"           => $itemNego["CDPRODFATURA"],
                "NRPARCFATURA"           => $itemNego["NRPARCFATURA"],
                "IDATIVO"                => "S",
                "NRORG"                  => $nrorg,
                "NRORGINCLUSAO"          => $nrorg,
                "CDOPERINCLUSAO"         => $cdoperador
            );
            $this->propostaFactory->inserirItemNegociacaoProposta($params);
            
            $params = array(
                "NRTIPOORCAMENTO" => $itemNego["NRTIPOORCAMENTO"],
                "NRITEMORCAMENTO" => $itemNego["NRITEMORCAMENTO"],
                "NRORG"           => $nrorg
            );
            $this->propostaFactory->atualizarEscopoItemProposta($params);

            $params = array(
                "NRSEQORCAMENTO"     => $nrseqorcamento,
                "NRITEMORCAMENTOPAI" => $itemNego["NRITEMORCAMENTO"],
                "NRORG"              => $nrorg
            );
            $itensFilhos = $this->propostaFactory->getItensFilhosItemProposta($params);
            
            foreach ($itensFilhos as $filho) {
                $nritnegociacaoproposta = $this->newCode->getCodeByTableName("FCT_ITNEGOCIACAOPROPOSTA", $nrorg, 1, 10, true);
                
                if (is_string($filho["VRTOTALITEMORCA"])) {
                    if (strpos($filho["VRTOTALITEMORCA"], ',') !== false) {
                        $filho["VRTOTALITEMORCA"] = (float) str_replace(',', '.', str_replace(".", "", $filho["VRTOTALITEMORCA"]));
                    }
                }
                    
                $params = array(
                    "NRITNEGOCIACAOPROPOSTA" => $nritnegociacaoproposta,
                    "NRNEGOCIACAOPROPOSTA"   => $nrnegociacaoproposta,
                    "NRITEMORCAMENTO"        => $filho["NRITEMORCAMENTO"],
                    "NRITEMORCAMENTOPAI"     => $filho["NRITEMORCAMENTOPAI"],
                    "VRTOTITEM"              => $filho["VRTOTALITEMORCA"],
                    "VRPERCDESCITEM"         => null,
                    "VRDESCITEM"             => null,
                    "VRTOTITEMCOMDESC"       => $filho["VRTOTALITEMORCA"],
                    "CDSERVORCA"             => $filho["CDSERVORCA"],
                    "CDPRODFATURA"           => $itemNego["CDPRODFATURA"],
                    "NRPARCFATURA"           => $itemNego["NRPARCFATURA"],
                    "IDATIVO"                => "S",
                    "NRORG"                  => $nrorg,
                    "NRORGINCLUSAO"          => $nrorg,
                    "CDOPERINCLUSAO"         => $cdoperador
                );
                $this->propostaFactory->inserirItemNegociacaoProposta($params);
            }

            $itemNegoAnterior = $itemNego;
        }

        foreach ($codNegociacoesInseridas as $cod) {
            $params = array(
                "NRNEGOCIACAOPROPOSTA" => $cod,
                "NRORG"                => $nrorg
            );
            $dataset = $this->propostaFactory->getSomaValorItensNegociacao($params);
            $valor = $dataset[0]["VRTOTAL"];
            
            $params = array(
                "VRTOTAL"              => $valor,
                "CDOPERULTATU"         => $cdoperador,
                "NRORGULTATU"          => $nrorg,
                "NRNEGOCIACAOPROPOSTA" => $cod,
                "NRORG"                => $nrorg
            );
            $this->propostaFactory->atualizarValorNegociacaoComSomaDosItens($params);
            
            $valorTotalNegociacaoComDesc = $valor;
            $percentualDesconto = 0;
            if (isset($desconto["value"])) {
                $valorDesconto = abs($desconto["value"]);
                $percentualDesconto = round($desconto["percent"],2);
                if ($valorDesconto > 0) {
                    $valorTotalNegociacaoComDesc = $valorTotalNegociacaoComDesc - $valorDesconto;
                    
                    $params = array(
                        "VRPERCDESCITEM"       => $percentualDesconto,
                        "VRDESCITEM"           => $valorDesconto,
                        "VRTOTITEMCOMDESC"     => $valorTotalNegociacaoComDesc,
                        "CDOPERULTATU"         => $cdoperador,
                        "NRORGULTATU"          => $nrorg,
                        "NRNEGOCIACAOPROPOSTA" => $cod,
                        "NRORG"                => $nrorg
                    );
                    $this->propostaFactory->atualizarValorNegociacaoComDescontoPM($params);
                }
            }
            
            $temParcela = ($parcelas) ? true : false;
            $this->inserirParcelaNegoProp($valorTotalNegociacaoComDesc, 30, $valorTotalNegociacaoComDesc, $cod, $nrorg, $cdoperador, 1, $temParcela, $parcelas, $codCliente);
        }
    }
    
    /**
     * Função privada utilizada para gerar um objeto contendo parâmetros necessários para efetivar a inserção da tabela 'FCT_PARCELANEGOPROP'.
     *@param int $dias Dias de faturamento
     *@param float $vrParcela Valor da parcela 
     *@param int $cod Codigo 
     *@param int $nrorg Número da organização
     *@param string $cdoperador Código do operador
     *@param int $qtdParcelas Quantidade de Parcelas
     *@return mixed[] Objeto contendo os dados para a inserção 
    */
    private function gerarParamsTabelaParcelaNegoProp($dias, $vrParcela, $cod, $nrorg, $cdoperador, $qtdParcelas){
        $nrparcelanegoprop = $this->newCode->getCodeByTableName("FCT_PARCELANEGOPROP", $nrorg, 1, 10, true);
        return array(
            "NRPARCELANEGOPROP"    => $nrparcelanegoprop,
            "NRNEGOCIACAOPROPOSTA" => $cod,
            "QTDIASFATURAMENTO"    => $dias,
            "NRDIAVENCIMENTO"      => null,
            "VRPARCELA"            => $vrParcela,
            "IDATIVO"              => "S",
            "NRORG"                => $nrorg,
            "NRORGINCLUSAO"        => $nrorg,
            "CDOPERINCLUSAO"       => $cdoperador,
            "NRPARCELA"            => $qtdParcelas
        );
    }
    
    /**
     * Função privada para inserir na tabela FCT_PARCELANEGOPROP
     *@param valorTotalNegociacaoComDesc Valor padrão da parcela
     *@param dias Dias de faturamento
     *@param vrParcela Valor da parcela 
     *@param cod Codigo 
     *@param nrorg Número da organização
     *@param cdoperador Código do operador
     *@param qtdParcelas Quantidade de Parcelas
     *@param temParcela tem parcelas (bool) 
     *@param parcelas Parcelas(obj)
     *@param codCliente Codigo do cliente 
     * 
    */
    private function inserirParcelaNegoProp($valorTotalNegociacaoComDesc, $dias, $vrParcela, $cod, $nrorg, $cdoperador, $qtdParcelas, $temParcela, $parcelas, $codCliente) {
        $semParcela = false;
        $indcAcessCntrldr = 1; //usada para controlar a posição do array a ser lida
        $params = $this->gerarParamsTabelaParcelaNegoProp($dias, $vrParcela, $cod, $nrorg, $cdoperador, $qtdParcelas);
        if($temParcela) { //Tem parcela dentro do params
            $qtdParcelas = count($parcelas); 
        }
        else{ //Não tem parcela mas vai tentar buscar de dentro do BD
            $qtparccust = $this->getQtParcCustCliente($codCliente, $nrorg);
            
            if ($qtparccust) {
                $qtdParcelas = $qtparccust;
                $vrParcela   = round($valorTotalNegociacaoComDesc / $qtdParcelas, 2);
            }
            else { // Não encontrou de jeito nenhum então vai inserir o parâmetro base
                $semParcela = true; 
                $this->integracaoFactory->inserirParcelaNegociacao($params);
            }
        }
        
        if (!$semParcela) { //config de variaveis sem parcela
            $cacheVlrNegocComDscnt = $params["VRPARCELA"]; 
            $vrParcelas = ($temParcela) ? 0 : round($valorTotalNegociacaoComDesc / $qtdParcelas, 2);
            $somaVrParcelas = 0;
            $qtdDiasFaturamento = 0;
            
            //Só rodara se tiver a parcela 
            for ($i = 1; $i <= $qtdParcelas ; $i++, $somaVrParcelas += $vrParcelas) {
                $nrparcelanegoprop           = $this->newCode->getCodeByTableName("FCT_PARCELANEGOPROP", $nrorg, 1, 10, true);
                $params["NRPARCELANEGOPROP"] = $nrparcelanegoprop;
                
                $qtdDiasFaturamento = ($temParcela) ? ($parcelas[$i-1]["billingDays"]) : (30+$qtdDiasFaturamento);
                
                if ($temParcela) {// só será feito no caso de existir a parcela
                    $vrParcela = round($parcelas[$i - $indcAcessCntrldr]["value"], 2);
                }
                
                if ($i == $qtdParcelas) { // sempre será feito
                    if ($temParcela)
                        $vrParcela = round($parcelas[$i - $indcAcessCntrldr]["value"], 2);
                    else 
                        $vrParcela = $cacheVlrNegocComDscnt - $somaVrParcelas;
                }
                    
                $params["QTDIASFATURAMENTO"] = $qtdDiasFaturamento;
                $params["VRPARCELA"]         = $vrParcela;
                $params["NRPARCELA"]         = $i;
                
                $this->integracaoFactory->inserirParcelaNegociacao($params);
                    
            }
        }
        
    }
    
    /*--- Inserir escopo de faturamento da Manutenção de Customização ---*/
    private function inserirNegociacaoValoresManutencaoCutomizacao($nrseqorcamento, $codCliente, $escopoManutCusto, $servOrcaManutCusto, $nrorg, $cdoperador, $valorTotal = null, $percDescontoMan = null, $valorDescontoMan = null, $valorTotalNegociacaoComDesc = null) {
        
        if (!is_null($valorDescontoMan)) {
            $valorDescontoMan = abs($valorDescontoMan);
        }

        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "NRORG" => $nrorg
        );

        $paramFatItemNegociacao = $this->integracaoFactory->getParametrosFaturamentoParaItensNegociacaoProposta($params, $escopoManutCusto, $servOrcaManutCusto);

        foreach ($paramFatItemNegociacao as $itemNego) {
            if ($itemNego["CDFILIALFATURA"] == null) {
                throw new \Exception('Operação bloqueada. <br/><br/>O parâmetro "Unidade para Faturamento" não foi encontrado para o item "' . $itemNego["NMITEMORCAMENTO"] . '". Favor conferir a parametrização de faturamento do Escopo.');
            }
            if ($itemNego["CDTIPOOPERFATURA"] == null) {
                throw new \Exception('Operação bloqueada. <br/><br/>O parâmetro "Tipo de Operação da Nota Fiscal" não foi encontrado para o item "' . $itemNego["NMITEMORCAMENTO"] . '". Favor conferir a parametrização de faturamento do Escopo.');
            }
            if ($itemNego["CDSERIENF"] == null) {
                throw new \Exception('Operação bloqueada. <br/><br/>O parâmetro "Série da Nota Fiscal" não foi encontrado para o item "' . $itemNego["NMITEMORCAMENTO"] . '". Favor conferir a parametrização de faturamento do Escopo.');
            }

            $competencia = array();
            if ($itemNego["IDTPFATURA"] != "P") {
                $params = array(
                    "NRTIPOORCAMENTO" => $itemNego["NRTIPOORCAMENTO"],
                    "CDSERVORCA"      => $itemNego["CDSERVORCA"],
                    "NRORG"           => $nrorg
                );
                $competencia = $this->propostaFactory->getCompetenciaItemNegociacao($params);
            }

            $itemNego["COMPETENCIA"] = print_r($competencia, true);
            
            $nrnegociacaoproposta = $this->newCode->getCodeByTableName("FCT_NEGOCIACAOPROPOSTA", $nrorg, 1, 10, true);
            $params = array(
                "NRNEGOCIACAOPROPOSTA" => $nrnegociacaoproposta,
                "NRSEQORCAMENTO"       => $nrseqorcamento,
                "NRTIPOORCAMENTO"      => $itemNego["NRTIPOORCAMENTO"],
                "IDTPFATURA"           => $itemNego["IDTPFATURA"],
                "CDFILIALFATURA"       => $itemNego["CDFILIALFATURA"],
                "NRPARCFATURA"         => $itemNego["NRPARCFATURA"],
                "CDTIPOOPERFATURA"     => $itemNego["CDTIPOOPERFATURA"],
                "CDSERIENF"            => $itemNego["CDSERIENF"],
                "CDEMPRESA"            => $itemNego["CDEMPRESA"],
                "CDINSCESTA"           => $itemNego["CDINSCESTA"],
                "CDINSCMUNI"           => $itemNego["CDINSCMUNI"],
                "VRTOTITEM"            => 0,
                "VRPERCDESCITEM"       => $percDescontoMan,
                "VRDESCITEM"           => $valorDescontoMan,
                "VRTOTITEMCOMDESC"     => 0,
                "IDATIVO"              => "S",
                "NRORG"                => $nrorg,
                "NRORGINCLUSAO"        => $nrorg,
                "CDOPERINCLUSAO"       => $cdoperador,
                "IDPREVCOBRFUT"        => $itemNego["IDPREVCOBRFUT"],
                "CDTRIBPRODPADR"       => $itemNego["CDTRIBPRODPADR"]
            );
            $this->propostaFactory->inserirNegociacaoProposta($params);
            
            foreach ($competencia as $comp) {
                $nrcompetencianegoprop = $this->newCode->getCodeByTableName("FCT_COMPETENCIANEGOPROP", $nrorg, 1, 10, true);
                $params = array(
                    "NRCOMPETENCIANEGOPROP" => $nrcompetencianegoprop,
                    "NRNEGOCIACAOPROPOSTA"  => $nrnegociacaoproposta,
                    "NRDIACOMPETINI"        => $comp["NRDIACOMPETINI"],
                    "NRDIACOMPETFIN"        => $comp["NRDIACOMPETFIN"],
                    "IDMESCOMPETINI"        => $comp["IDMESCOMPETINI"],
                    "IDMESCOMPETFIN"        => $comp["IDMESCOMPETFIN"],
                    "QTDIASAPURACAO"        => $comp["QTDIASAPURACAO"],
                    "IDTPCALCVENC"          => $comp["IDTPCALCVENC"],
                    "NRDIACALCVENC"         => $comp["NRDIACALCVENC"],
                    "IDMESCALCVENC"         => $comp["IDMESCALCVENC"],
                    "QTDIASFATUVENC"        => $comp["QTDIASFATUVENC"],
                    "IDATIVO"               => "S",
                    "NRORG"                 => $nrorg,
                    "NRORGINCLUSAO"         => $nrorg,
                    "CDOPERINCLUSAO"        => $cdoperador
                );
                $this->propostaFactory->inserirCompetenciaNegociacaoProposta($params);
            }                
            
            if (is_string($itemNego["VRTOTALITEMORCA"])) {
                if (strpos($itemNego["VRTOTALITEMORCA"], ',') !== false) {
                    $itemNego["VRTOTALITEMORCA"] = (float) str_replace(',', '.', str_replace(".", "", $itemNego["VRTOTALITEMORCA"]));
                }
            }

            $nritnegociacaoproposta = $this->newCode->getCodeByTableName("FCT_ITNEGOCIACAOPROPOSTA", $nrorg, 1, 10, true);
            $params = array(
                "NRITNEGOCIACAOPROPOSTA" => $nritnegociacaoproposta,
                "NRNEGOCIACAOPROPOSTA"   => $nrnegociacaoproposta,
                "NRITEMORCAMENTO"        => $itemNego["NRITEMORCAMENTO"],
                "NRITEMORCAMENTOPAI"     => $itemNego["NRITEMORCAMENTOPAI"],
                "VRTOTITEM"              => $itemNego["VRTOTALITEMORCA"],
                "VRPERCDESCITEM"         => null,
                "VRDESCITEM"             => null,
                "VRTOTITEMCOMDESC"       => $itemNego["VRTOTALITEMORCA"],
                "CDSERVORCA"             => $itemNego["CDSERVORCA"],
                "CDPRODFATURA"           => $itemNego["CDPRODFATURA"],
                "NRPARCFATURA"           => $itemNego["NRPARCFATURA"],
                "IDATIVO"                => "S",
                "NRORG"                  => $nrorg,
                "NRORGINCLUSAO"          => $nrorg,
                "CDOPERINCLUSAO"         => $cdoperador
            );
            $this->propostaFactory->inserirItemNegociacaoProposta($params);
        }
        
        $params = array(
            "NRNEGOCIACAOPROPOSTA" => $nrnegociacaoproposta,
            "NRORG"                => $nrorg
        );
        
        $dataset = $this->propostaFactory->getSomaValorItensNegociacao($params);
    
        $valor = $dataset[0]["VRTOTAL"];
       
        $params = array(
            "VRTOTAL"              => $valor,
            "VRDESCITEM"           => $valorDescontoMan,
            //"VRTOTITEMCOMDESC"     => is_null($valorTotalNegociacaoComDesc)? $valor : $valorTotalNegociacaoComDesc,
            "VRTOTITEMCOMDESC"     => is_null($valorTotalNegociacaoComDesc)? $valor-$valorDescontoMan : $valorTotalNegociacaoComDesc,
            "CDOPERULTATU"         => $cdoperador,
            "NRORGULTATU"          => $nrorg,
            "NRNEGOCIACAOPROPOSTA" => $nrnegociacaoproposta,
            "NRORG"                => $nrorg
        );

        $this->propostaFactory->atualizarValorNegociacaoComSomaDosItensPM($params);
        
    }
    
    //aqui
    public function aprovarContrato($nrseqorcamento, $nrorg, $cdoperador, $isConnected = false) {
        if(!$isConnected) {
            $this->setConnection($nrorg);
            $this->integracaoFactory = new IntegracaoPmFactory($this->connection);
            $this->contratoFactory   = new Sm00004000ContratoFactory($this->entityManager);
            $this->newCode           = new NewCode($this->connection, "NOVO_CODIGO_SC");
        }
        
        $this->connection->beginTransaction();
        try {
            $params = array(
                "NRSEQORCAMENTO" => $nrseqorcamento,
                "NRORG"          => $nrorg
            );
            $contratos = $this->integracaoFactory->getContratosLigadosAProposta($params);
            foreach($contratos as $contrato) {
                if($contrato['IDATIVO'] == 'S' && $contrato['IDSITUACAOCONTRATO'] == 'P'){
                    $nrseqcontrato = $contrato["NRSEQCONTRATO"];
                    $this->salvarInfoPrecoBaseContrato($nrseqcontrato, $nrorg, $cdoperador);
                    $this->inserirAprovacaoContrato($nrseqcontrato, $nrorg, $cdoperador);
                    
                    $procedure = new FctGerarParcelasFatura($this->connection);
                    $procedure->executar($nrseqcontrato, $nrorg, $cdoperador);
                }
            }
            //ver se é necessário gerar um novo contrato automatico para faturamento de servicos
            //arthur
            $this->connection->commit();
        } catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function salvarInfoPrecoBaseContrato($nrseqcontrato, $nrorg, $cdoperador) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );
        
        $medidasContrato = $this->contratoFactory->getValorVariacoesPrecoContrato($params);
        if($medidasContrato[0]["QTMEDIDAS"] == 0) {

            $moedaIdiomaCont = $this->contratoFactory->getMoedaIdiomaContrato($params);

            $moeda = $moedaIdiomaCont[0]["NRMOEDA"];
            $paramsVar = array(
                "NRSEQCONTRATO" => $nrseqcontrato,
                "NRORG"         => $nrorg,
                "NRMOEDA"       => $moeda
            );
            
            $variacoes = $this->contratoFactory->getVariacaoPrecoBaseContrato($paramsVar);
            foreach($variacoes as $variacao) {
                $nrvarprecobasecont = $this->newCode->getCodeByTableName("FCT_VARPRECBASESERVCONT", $nrorg, 1, 10, true);
                $params = array(
                    "NRVARPRECBASESERVCONT" => $nrvarprecobasecont,
                    "NRSEQCONTRATO"         => $nrseqcontrato,
                    "CDSERVORCA"            => $variacao["CDSERVORCA"],
                    "CDTPMEDIDASERV"        => $variacao["CDTPMEDIDASERV"],
                    "CDTPMEDIDAVARIAVEL"    => $variacao["CDTPMEDIDAVARIAVEL"],
                    "QTMEDIDAINIVARIACAO"   => $variacao["QTMEDIDAINIVARIACAO"],
                    "QTMEDIDAFIMVARIACAO"   => $variacao["QTMEDIDAFIMVARIACAO"],
                    "QTFRANQUIA"            => $variacao["QTFRANQUIA"],
                    "VRPRECOVARIACAO"       => $variacao["VRPRECOVARIACAO"],
                    "IDMULTIPVARIACAO"      => $variacao["IDMULTIPVARIACAO"],
                    "NRORG"                 => $nrorg,
                    "NRORGINCLUSAO"         => $nrorg,
                    "IDATIVO"               => "S",
                    "CDOPERINCLUSAO"        => $cdoperador
                );
                $this->contratoFactory->inserirDadosVariacaoPrecoBaseContrato($params);
            }
        }
    }
    
    public function inserirAprovacaoContrato($nrseqcontrato, $nrorg, $cdoperador) {
        $nrseqaprovcont = $this->newCode->getCodeByTableName("FCT_APROVACONTRATO", $nrorg, 1, 10, true);
        $params = array (
            "NRSEQAPROVCONT" => $nrseqaprovcont,
            "NRSEQCONTRATO"  => $nrseqcontrato,
            "IDAPROCCONTR"   => "A",
            "NRORG"          => $nrorg,
            "OPERADOR"       => $cdoperador
        );
        
        $this->integracaoFactory->inserirInfoAprovacao($params);
        
        $params = array(
            "NRSEQCONTRATO"      => $nrseqcontrato,
            "NRORG"              => $nrorg,
            "CDOPERULTATU"       => $cdoperador
        );
        
        $this->integracaoFactory->alterarSituacaoContrato($params);
    }
    
    /*********************************************************************
     *      Requerimentos realizados via SOLICITAÇÕES (Delphi)
    *********************************************************************/
    public function salvarPropostaSolicitacoes($nrorg) {
        $this->setConnection($nrorg);
        $this->integracaoFactory = new IntegracaoPmFactory($this->connection);
        $this->newCode           = new NewCode($this->connection, "NOVO_CODIGO_SC");

        $params = array(
            "NRORG"      => $nrorg,
            "CDFASETRAM" => $this->idSolicitacaoAguardProp
        );
        
        $orcamentos   = array();
        $solicitacoes = $this->integracaoFactory->getInfoSolicitacoesAguardadoProposta($params);
        foreach ($solicitacoes as $solicitacao) {
            $this->connection->beginTransaction();
            try {
                $solicitacaoComContratoAprovado = false;
                
                if ($solicitacao["CDCLIENTE"] !== $this->clienteSignatarioPrestador) {
                    
                    if (!empty($solicitacao["NRSEQORCAMENTO"])) {
                        
                        $params = array(
                            "NRSEQORCAMENTO" => $solicitacao["NRSEQORCAMENTO"],
                            "NRORG"          => $nrorg
                        );
                        $contratos = $this->integracaoFactory->getContratosLigadosAProposta($params);
                        
                        if (count($contratos) > 0) {
                            foreach ($contratos as $contrato) {
                                if ($contrato["IDSITUACAOCONTRATO"] != "A") {
                                    $this->excluirContrato($contrato["NRSEQCONTRATO"], $nrorg);
                                }
                                else {
                                    $solicitacaoComContratoAprovado = true;
                                    
                                    //dar update na tabela LOGFASETRAM colocando o campo DSALFASETRAM como: "ERRO: Não foi possível gerar proposta no SM pois essa solicitação já possui um contrato aprovado.".
                                    $msgErro = "Não foi possível gerar proposta no SM pois essa solicitação já possui um contrato aprovado (código " . $contrato["NRCONTRATOFCT"] . ").";
                                    $dataAtualString = date_create()->format("d/m/Y H:i:s");
                                    
                                    $params = array(
                                        "DSALFASETRAM"  => "$dataAtualString - ERRO: $msgErro",
                                        "NRSOLISERV"    => $solicitacao["NRSOLISERV"],
                                        "CDFILIAL"      => $solicitacao["CDFILIAL"],
                                        "CDFASETRAM"    => $this->idSolicitacaoAguardProp,
                                        "NRCONTRATOFCT" => $contrato["NRCONTRATOFCT"]
                                    );
                                    $this->integracaoFactory->atualizarLogFaseTramPropostaNaoGerada($params);
        
                                    //throw new \Exception($msgErro);
                                    $message = "Solicitação: " . $solicitacao["NRSOLISERV"] . " | ERRO: \n". $msgErro . "\n";
                                }
                            }
                        }
                        
                        if (!$solicitacaoComContratoAprovado) {
                            $this->atualizarPropostaCancelamento($solicitacao["NRSEQORCAMENTO"], self::OPERADOR_TEKNISA, $nrorg);
                        }
                    }
                    
                    if (!$solicitacaoComContratoAprovado) {
                        $nrsoliserv    = $solicitacao["NRSOLISERV"];
                        $cdcliente     = $solicitacao["CDCLIENTE"];
                        $cdfilial      = $solicitacao["CDFILIAL"];
                        $descricao     = $solicitacao["DSREFESOLI"];
                        $codproduto    = $solicitacao["CDPRODGRUP"];
                        $cdoperultfase = $solicitacao["CDOPERULTFASE"];
                        $operSoli      = $solicitacao["CDOPERSOLI"];
                        
                        $desconto = array(
                            "percent" => $solicitacao["VRPEDESORCAM"],
                            "value"   => $solicitacao["VRDESCORCAM"]
                        );
                        
                        $nminterlocutor    = null;
                        $emailInterlocutor = null;
                        $interlocutor      = $this->getDadosInterlocutor($operSoli, $nrorg);
                        if(count($interlocutor) > 0) {
                            $emailInterlocutor = $interlocutor[0]["DSEMAILOPER"];
                            $nminterlocutor    = $interlocutor[0]["NMOPERADOR"];
                        }
                        
                        $nomeProduto = $this->getNomeProdutoSolicitacao($codproduto, $nrorg);
                        $itens       = $this->getItensSolicitacao($nrsoliserv, $nrorg, $cdfilial);
                        if (count($itens) == 0) {
                            throw new \Exception("Não foram encontrados itens a serem inseridos na proposta. Favor conferir as tabelas ORCASOLICITA e ITEMORCAPROD_FCT_ITEMORCAMENTO.");
                        }
                        $parcelas = $this->getParcelasSolicitacao($nrsoliserv, $nrorg, $cdfilial); 

                        $params = array(
                            "NRSOLISERV" => $nrsoliserv,
                            "CDFILIAL"   => $cdfilial,
                            "NRORG"      => $nrorg
                        );
                        
                        $DescSolGestDem = $this->integracaoFactory->getDescontoSolGestaoDemanda($params); 
                        if (count($DescSolGestDem) > 0) {
                            $percDescontoMan =  $DescSolGestDem[0]["MAINTENANCE_PERCENT"];
                            $valorDescontoMan = $DescSolGestDem[0]["MAINTENANCE_VALUE"];
                        }
                        else {
                            $percDescontoMan = null; 
                            $valorDescontoMan = null;
                        }

                        $nrseqorcamento = $this->prepararInserirProposta($nrorg, $cdoperultfase, $cdcliente, $codproduto,
                            $descricao, $emailInterlocutor, $nminterlocutor, $itens, $desconto, $parcelas, true,
                            $nomeProduto, null, $nrsoliserv, null, $percDescontoMan, $valorDescontoMan, null);
                        
                        /* Função retirada pois já está sendo vinculado o NRSOLISERV na proposta na função "prepararInserirProposta".
                        $params = array(
                            "NRSEQORCAMENTO" => $nrseqorcamento,
                            "NRSOLISERV"     => $nrsoliserv,
                            "NRORG"          => $nrorg
                        );
                        $this->integracaoFactory->vincularSolicitacaoAProposta($params);
                        */
                    
                        array_push($orcamentos, array(
                        "NRSOLISERV"     => $nrsoliserv,
                        "NRSEQORCAMENTO" => $nrseqorcamento
                        
                        ));
                        
                        $message = "Solicitação: " . $nrsoliserv . " | Proposta gerada: " . $nrseqorcamento . "\n";
                    }
                }
                else {
                    $message = "Solicitação: " . $solicitacao["NRSOLISERV"] . " | Fase de tramitação atualizada. Não foi criada proposta no SM (cliente Teknisa)\n";
                    $this->alterarSituacaoSolicitacao($solicitacao["NRSOLISERV"], $nrorg, $solicitacao["CDFILIAL"], $this->idSolicitacaoAguardProp, $this->idSolicitacaoAguardCli,  $solicitacao["CDOPERRESP"], true);
                }
                
                $this->connection->commit();
                
                echo $message;
            }
            catch (\Exception $e) {
                echo "Solicitação: " . $solicitacao["NRSOLISERV"] . " | ERRO: \n". $e->getMessage() . "\n";
                $this->connection->rollback();
            }
        }
    }
    
    public function getDadosInterlocutor($cdoperador, $nrorg) {
        $params = array(
            "CDOPERADOR" => $cdoperador,
            "NRORG"      => $nrorg
        );

        return $this->integracaoFactory->getDadosInterlocutor($params);
    }
    
    public function getItensSolicitacao($nrsoliserv, $nrorg, $cdfilial) {
        $params = array(
            "NRSOLISERV" => $nrsoliserv,
            "CDFILIAL"   => $cdfilial,
            "NRORG"      => $nrorg
        );
        
        $itens   = array();
        $dataset = $this->integracaoFactory->getItensSolicitacao($params);
        foreach($dataset as $item) {
            $itens[] = array(
                "quantity"     => $item["QUANTITY"],
                "totalValue"   => $item["TOTALVALUE"],
                "unitaryValue" => $item["UNITARYVALUE"],
                "measurement"  => $item["MEASUREMENT"],
                "id"           => $item["ID"]
            );
        }
        
        return $itens;
    }
    
    public function getParcelasSolicitacao($nrsoliserv, $nrorg, $cdfilial) {
        $params = array(
            "NRSOLISERV" => $nrsoliserv,
            "CDFILIAL"   => $cdfilial,
            "NRORG"      => $nrorg
        );
        
        $dataset = $this->integracaoFactory->getParcelasSolicitacao($params);
        return count($dataset) > 0 ? $dataset : null;
    }
    
    public function getNomeProdutoSolicitacao($codproduto, $nrorg) {
        $params = array(
            "CDPRODUTO" => $codproduto,
            "NRORG"     => $nrorg
        );
        
        return $this->integracaoFactory->getNomeProdutoSolicitacao($params);
    }
    
    public function getFilialSolicitacao($nrsoliserv, $nrorg) {
        $params = array(
            "NRSOLISERV" => $nrsoliserv,
            "NRORG"      => $nrorg
        );
        
        $filial = $this->integracaoFactory->getFilialSolicitacao($params);
        return (count($filial) > 0) ? $filial[0]["CDFILIAL"] : null;
    }
    
    public function alterarSituacaoSolicitacao($nrsoliserv, $nrorg, $cdfilial, $cdfasetram, $cdfastramdes, $operresp, $clienteTeknisa = false) {
        
        $params = array(
            "NRORG"      => $nrorg,
            "NRSOLISERV" => $nrsoliserv,
            "CDFILIAL"   => $cdfilial,
            "CDFASETRAM" => $cdfastramdes
        );
        
        $this->integracaoFactory->alterarSituacaoSolicitacao($params);
        $this->inserirLogAlteracaoFaseTram($nrsoliserv, $nrorg, $cdfilial, $cdfasetram, $cdfastramdes, $operresp, $clienteTeknisa);
    }
    
    public function aprovarContratoSolicitacoes($nrorg) {
        $this->setConnection($nrorg);
        $this->integracaoFactory = new IntegracaoPmFactory($this->connection);
        $this->contratoFactory   = new Sm00004000ContratoFactory($this->entityManager);
        $this->newCode           = new NewCode($this->connection, "NOVO_CODIGO_SC");
        
        $params = array(
            "NRORG"      => $nrorg,
            "CDFASETRAM" => $this->idSolicitacaoAguardAprov
        );
        
        $solicitacoes = $this->integracaoFactory->getInfoSolicitacoesAguardadoProposta($params);
        foreach($solicitacoes as $solicitacao) {
            $this->connection->beginTransaction();
            try {
                $nrsoliserv = $solicitacao["NRSOLISERV"];
                $cdfilial   = $solicitacao["CDFILIAL"];
                
                $params = array(
                    "NRSOLISERV" => $nrsoliserv,
                    "NRORG"      => $nrorg
                );
                
                $propostasRelacionadas = array();
                $orcamentos = $this->integracaoFactory->getOcamentoSolicitacao($params);
                foreach($orcamentos as $orcamento) {
                    $nrseqorcamento = $orcamento["NRSEQORCAMENTO"];
                    $propostasRelacionadas[] = $nrseqorcamento;
                    
                    $this->aprovarContrato($nrseqorcamento, $nrorg, self::OPERADOR_TEKNISA, true);
                }
                
                $this->alterarSituacaoSolicitacao($nrsoliserv, $nrorg, $cdfilial, $this->idSolicitacaoAguardAprov, $this->idSolicitacaoLiberada, $solicitacao["CDOPERRESP"]);
         
                $this->connection->commit();
                
                $propostasRelacionadas = implode(",", $propostasRelacionadas);
                echo "Solicitação: " . $nrsoliserv . " | Proposta relacionada: ". $propostasRelacionadas . " | Contratos aprovados com sucesso" . "\n";
            }
            catch (\Exception $e) {
                echo "Solicitação: " . $solicitacao["NRSOLISERV"] . " | ERRO: \n". $e->getMessage() . "\n";
                $this->connection->rollback();
            }
        }
    }
    
    public function inserirLogAlteracaoFaseTram($nrsoliserv, $nrorg, $cdfilial, $cdfasetram, $cdfastramdes, $operresp, $clienteTeknisa = false) {
        $txtacaofasetram = $this->integracaoFactory->getTxAcaoFaseTramitacao(
            array(
                "CDFASETRAM"   => $cdfasetram,
                "CDFASTRAMDES" => $cdfastramdes
            )
        );
        
        $dsalteracao = array(
            $this->idSolicitacaoAguardCli => $clienteTeknisa ? "Fase de tramitação atualizada. Não foi criada proposta no SM (cliente Teknisa)." : "Criação da proposta de customização pelo sistema SM.",
            $this->idSolicitacaoLiberada  => "Liberação do contrato para faturamento pelo sistema SM."
        );
        
        $params = array(
            "CDFILIAL"       => $cdfilial,
            "NRSOLISERV"     => $nrsoliserv,
            "CDFASETRAM"     => $cdfastramdes,
            "CDOPERADOR"     => self::OPERADOR_TEKNISA,
            "DSALFASETRAM"   => $dsalteracao[$cdfastramdes],
            "TXACAOLFTRAM"   => $txtacaofasetram,
            "CDOPERRESP"     => $operresp,
            "NRORG"          => $nrorg,
            "NRORGINCLUSAO"  => $nrorg,
            "CDOPERINCLUSAO" => "1"
        );
        
        $this->integracaoFactory->inserirLogAlteracaoFaseTram($params);
    }
    
    public function getQtParcCustCliente($codCliente, $nrorg) {
        $params = array(
            "CDCLIENTE" => $codCliente,
            "NRORG"     => $nrorg
        );
        
        $dataset = $this->integracaoFactory->getQtParcCustCliente($params);
        return (count($dataset) > 0) ? $dataset[0]["QTPARCCUST"] : null;
    }

    /**
     * @param $nomeProduto
     * @param $issueId
     * @param $nrsoliserv
     * @return string
     */
    private function createReferencia($nomeProduto, $issueId, $nrsoliserv)
    {
        $dsreferenciaorca = "Customização do Produto " . $nomeProduto;
        if ($issueId) {
            $dsreferenciaorca = "Issue " . $issueId . " - " . $dsreferenciaorca;
        } else if ($nrsoliserv) {
            $dsreferenciaorca = "Solicitação " . $nrsoliserv . " - " . $dsreferenciaorca;
        }
        return $dsreferenciaorca;
    }
    
    private function atualizarPropostaCancelamento($nrseqorcamento, $cdoperador, $nrorg) {
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
            "CDOPERULTATU"   => $cdoperador,
            "NRORGULTATU"    => $nrorg,
            "NRORG"          => $nrorg
        );
        
        $this->integracaoFactory->atualizarPropostaCancelamento($params);
    }

    private function excluirContrato($nrseqcontrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );

        $dataset = $this->integracaoFactory->getAnexosContrato($params);
        if (count($dataset) > 0) {
            $this->setZhFileUploader($nrorg);

            foreach ($dataset as $row) {
                $caminho = $row["DSCAMINHO"];
                if (!empty($caminho)) {
                    $this->zhFileUploader->deleteFile($caminho);
                }
            }
        }

        $this->integracaoFactory->excluirAnexosContrato($params);
        $this->integracaoFactory->excluirContrato($params);
    }

    private function getIdiomaMoedaParceiro($nrparcnegocio) {
        $params = [
            "NRPARCNEGOCIO" => $nrparcnegocio
        ];

        $dataset = $this->integracaoFactory->getIdiomaMoedaParceiro($params);
        return $dataset[0];
    }
}

?>